!!!!!!!!!!
!
!fichier g2s_segArc.md9
!
!  module segArc
!  #############
! C'est dans ce module que sont gerees toutes les fonctions relatives aux
! elements geometriques.
! Une structure "t_segArc" est definie, avec tous les champs necessaires aux
! calculs. On stoque par ailleurs ces elements dans un tableau global de
! pointeurs "tabSegArc".
!
! variable globale:
!  - tabSegArc : le tableau de travail principal
!
! fonctions du module:
!  - initializeTabSegArc : mise a zero des tableaux
!  - destroyTabSegArc : liberation de la memoire
!  - createSeg : constructeur de segment
!  - createArc : constructeur de cercle ou d'arc de cercle
!  - giveOrigine : retourne les coordonnes de l'origine d'un segArc
!  - giveExtremite : retourne les coordonnes de l'extremite d'un segArc
!  - extremitesArc : determination des points extremites d'un arc
!  - estColieaire : test de la colinearite de 2 segments
!  - isAngleInArc : test l'appartenance d'un angle au domaine d'un arc
!  - isIn : test l'appartenance d'un point, sur le support d'un segment ou
!           d'un arc, au segment ou a l'arc
!  - isSameWay : test si deux segments alignes ont meme orientation
!  - turnBackSide : retourne le segment donne en entree
!  - adjustMix : ajuste les milieux a droite et a gauche d'un segment, par
!                rapport a ceux d'un segment de reference le contenant
!  - interSgAr : calcul les points d'untersection entre un segment et un arc
!  - interSgSg : calcul les points d'untersection entre deux segments
!  - interArcArc : calcul les points d'untersection entre deux arcs
!  - translateAndTurnTriSg : effectue une translation-rotation d'un cote
!                            de triangle (fonction tres specifique)
!  - giveExtremalsAngles : donne les angles d'incidences des extrmites d'un
!                          element, par rapport a un point
!  - drawSegArc : affiche le tableau des elements dans un fichier postscript
!  - coupeCercle : intersecte les couronnes et les clusters
!  - splitSegsForSect : coupe l'encadrement d'une cellule rectangulaire en cas
!                       de sectorisation
!  - majSectori : met a jour la donnee sur le secteur des elements
!  - addSegsAndClean : ajout de nouveaux segments lors de la mise cote a cote
!                      des differentes cellules, et elimination des doublons
!  - cutClusters : prise en compte des intersections couronnes/cluster, et
!                  elimination de elements internes aux clusters
!
!
!!!!!!!!!!!

module segArc
  use constUtiles
  use constType
  use derivedPSPLOT

  implicit none

  type t_segArc
     integer           :: typ   !1=segment, 2=arc de cercle
     double precision  :: x,y   !origine si 1 , centre si 2
     double precision  :: dx,dy !extremite si 1
     double precision  :: r     !rayon si 2
     double precision  :: a,b   !angle debut et fin si 2 (entre -pi et pi)
     integer           :: mixg  =0!numero de couronne a gauche ou interieur
     integer           :: mixd  =0!numero de couronne a droite ou exterieur
     integer           :: nodeg =0!node a gauche ou interieur
     integer           :: noded =0!node a droite ou exterieur
     integer           :: indCellPg=0 !cellulePlaced d'origine a gauche
     integer           :: indCellPd=0 !cellulePlaced d'origine a droite
     integer           :: sectg=0 !numero du secteur a gauche dans la cellule
     integer           :: sectd=0 !numero du secteur a droite dans la cellule
     integer           :: neutronicMixg=0 !milieux neutronique a gauche
     integer           :: neutronicMixd=0 !milieux neutronique a droite
  end type t_segArc

  integer,parameter :: tseg=1 , tcer=2 , tarc=3 , fooMix=-99 , fooNode=-999 , &
       & tRec=1 , tHex=2 , tTri=3

  type segArcArray
     type(t_segArc),pointer :: p
  end type segArcArray

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !!  decalration d'une variable tableau globale
  !!
  !variable globale de type tableau d'elements geometriques
  type(segArcArray),dimension(:),pointer,save :: tabSegArc
  !!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  type segArcArrayBis
     type(t_segArc),pointer :: p
     logical                :: keep
  end type segArcArrayBis

  type segArcArrayTer
     type(t_segArc),pointer :: p
     logical                :: keep
     logical                :: cl   !condition limite ou cluster selon les cas
  end type segArcArrayTer

contains

  subroutine initializeTabSegArc(dimTabSegArc)
    implicit none
    integer,intent(in) :: dimTabSegArc
    integer :: i

    allocate(tabSegArc(dimTabSegArc))
    do i = 1,dimTabSegArc
       nullify(tabSegArc(i)%p)
    end do
  end subroutine initializeTabSegArc

  subroutine destroyTabSegArc(szSA)
    implicit none
    integer,intent(in) :: szSA
    integer :: i

    do i = 1,szSA
       deallocate(tabSegArc(i)%p)
       nullify(tabSegArc(i)%p)
    end do
    deallocate(tabSegArc)
    nullify(tabSegArc)
  end subroutine destroyTabSegArc

  function createSeg(ox,oy,ex,ey,mg,md)
    implicit none
    double precision,intent(in) :: ox,oy,ex,ey
    integer,intent(in)          :: mg,md
    type(t_segArc)              :: createSeg

    createSeg%typ   = tseg
    createSeg%x     = ox
    createSeg%y     = oy
    createSeg%dx    = ex
    createSeg%dy    = ey
    createSeg%r     = 0.d0
    createSeg%a     = 0.d0
    createSeg%b     = 0.d0
    createSeg%mixg  = mg
    createSeg%mixd  = md
    createSeg%nodeg = fooNode
    createSeg%noded = fooNode
    createSeg%indCellPg = 0
    createSeg%indCellPd = 0
    createSeg%sectg = 0
    createSeg%sectd = 0
    createSeg%neutronicMixg = 0
    createSeg%neutronicMixd = 0
  end function createSeg

  function createArc(cx,cy,r,a,b,mi,me)
    double precision,intent(in) :: cx,cy,r,a,b
    integer,intent(in)          :: mi,me
    type(t_segArc)              :: createArc

    createArc%a     = a
    createArc%b     = b
    if (isEqual(createArc%a,createArc%b)) then
       createArc%typ  = tcer
    else
       createArc%typ  = tarc
    end if
    createArc%x     = cx
    createArc%y     = cy
    createArc%dx    = 0.d0
    createArc%dy    = 0.d0
    createArc%r     = r
    createArc%mixg  = mi
    createArc%mixd  = me
    createArc%nodeg = fooNode
    createArc%noded = fooNode
    createArc%indCellPg = 0
    createArc%indCellPd = 0
    createArc%sectg = 0
    createArc%sectd = 0
    createArc%neutronicMixg = 0
    createArc%neutronicMixd = 0
  end function createArc

  subroutine giveOrigine(sa,xx,yy)
    implicit none
    type(t_segArc),intent(in)    :: sa
    double precision,intent(out) :: xx,yy

    select case(sa%typ)
    case(tseg)
       xx = sa%x ; yy = sa%y
    case(tarc)
       xx = sa%x+sa%r*cos(sa%a) ; yy=sa%y+sa%r*sin(sa%a)
    case(tcer)
       xx = sa%x+sa%r ; yy = sa%y
    end select
  end subroutine giveOrigine

  subroutine giveExtremite(sa,xx,yy)
    implicit none
    type(t_segArc),intent(in)    :: sa
    double precision,intent(out) :: xx,yy

    select case(sa%typ)
    case(tseg)
       xx = sa%dx ; yy = sa%dy
    case(tarc)
       xx = sa%x+sa%r*cos(sa%b) ; yy=sa%y+sa%r*sin(sa%b)
    case(tcer)
       xx = sa%x+sa%r ; yy = sa%y
    end select
  end subroutine giveExtremite

  subroutine extremitesArc(ar,pt1x,pt1y,pt2x,pt2y)
    implicit none
    type(t_segArc),intent(in)    :: ar
    double precision,intent(out) :: pt1x,pt1y,pt2x,pt2y

    pt1x=ar%x+ar%r*cos(ar%a) ; pt1y=ar%y+ar%r*sin(ar%a)
    pt2x=ar%x+ar%r*cos(ar%b) ; pt2y=ar%y+ar%r*sin(ar%b)
  end subroutine extremitesArc

  function estColineaire(sa1,sa2)
    type(t_segArc),intent(in) :: sa1,sa2
    logical                   :: estColineaire

    if (sa1%typ/=tseg .or. sa2%typ/=tseg) then
       estColineaire = .false.
    else
       estcolineaire = estcoli(sa1%dx-sa1%x,sa1%dy-sa1%y,sa2%dx-sa2%x,sa2%dy-sa2%y)
    end if
  end function estColineaire

  function isAngleInArc(a,arc)
    double precision,intent(in) :: a
    type(t_segArc),intent(in)   :: arc
    integer                     :: isAngleInArc

    double precision :: angl

    !dit si un angle est sur sur l'interval d'un arc : 0 -> pas dessus,
    ! 1 -> c'est l'origine, 2 -> entre les deux, 3 -> c'est l'extremite
    if (arc%typ==tcer) then
       isAngleInArc = 2
       return
    end if

    angl = angleNormal(a)
    if (isEqualConst(arc%a,angl)) then
       isAngleInArc = 1
    else if (isEqualConst(arc%b,angl)) then
       isAngleInArc = 3 
    else if (arc%a<arc%b) then
       if((arc%a<angl).and.(angl<arc%b)) then
          isAngleInArc = 2
       else
          isAngleInArc = 0
       end if
    else
       if((arc%b<angl).and.(angl<arc%a)) then
          isAngleInArc = 0
       else
          isAngleInArc = 2
       end if
    end if
  end function isAngleInArc

  function isIn(ptx,pty,sa)
    double precision,intent(in) :: ptx,pty
    type(t_segArc),intent(in)   :: sa
     integer                     :: isIn

    double precision :: angl

    !dit si un point est sur un segment ou un arc : 0 -> pas dessus,
    ! 1 -> c'est l'origine, 2 -> entre les deux, 3 -> c'est l'extremite
    !ATENTION:le point doit etre deja sur la droite ou le cercle d'appuis
    if (sa%typ==tseg) then
       if (isEqualConst(sa%x,ptx) .and. isEqualConst(sa%y,pty)) then
          isIn = 1
       else if (isEqualConst(sa%dx,ptx) .and. isEqualConst(sa%dy,pty)) then
          isIn = 3
       else if (((sa%x-ptx)*(sa%dx-ptx)+(sa%y-pty)*(sa%dy-pty))<0) then
          isIn = 2
       else
          isIn = 0
       end if
    else
       angl = calculeAngle(sa%x,sa%y,ptx,pty)
       isIn = isAngleInArc(angl,sa)
    end if
  end function isIn

  function isSameWay(sg1,sg2)
    implicit none
    type(t_segArc),intent(in) :: sg1,sg2
    logical                   :: isSameWay
    !dit si 2 segments alignes ont meme orientation

    isSameWay=(((sg1%dx-sg1%x)*(sg2%dx-sg2%x)+(sg1%dy-sg1%y)*(sg2%dy-sg2%y))>0)
  end function isSameWay

  function turnBackSide(sg)
    implicit none
    type(t_segArc),intent(inout) :: sg
    type(t_segArc)               :: turnBackSide
    !tourne le sens du segment en entree

    turnBackSide=createSeg(sg%dx,sg%dy,sg%x,sg%y,sg%mixd,sg%mixg)
    turnBackSide%nodeg=sg%noded
    turnBackSide%noded=sg%nodeg
    turnBackSide%indCellPg=sg%indCellPd
    turnBackSide%indCellPd=sg%indCellPg
    turnBackSide%sectg=sg%sectd
    turnBackSide%sectd=sg%sectg
    turnBackSide%neutronicMixg=sg%neutronicMixd
    turnBackSide%neutronicMixd=sg%neutronicMixg
    sg=turnBackSide
  end function turnBackSide

  subroutine adjustMix(sgToAdjust,sgOfRef)
    implicit none
    type(t_segArc),intent(inout) :: sgToAdjust
    type(t_segArc),intent(in)    :: sgOfRef
    !ajuste les milieux a droite et a gauche de sgToAdjust
    !par rapport a ceux de sgOfRef (sgToAdjust doit etre inclus dans sgOfRef)
    if (sgOfRef%mixg/=fooMix) then
       sgToAdjust%mixg          = sgOfRef%mixg
       sgToAdjust%indCellPg     = sgOfRef%indCellPg
       sgToAdjust%sectg         = sgOfRef%sectg
       sgToAdjust%neutronicMixg = sgOfRef%neutronicMixg
    end if
    if (sgOfRef%mixd/=fooMix) then
       sgToAdjust%mixd          = sgOfRef%mixd
       sgToAdjust%indCellPd     = sgOfRef%indCellPd
       sgToAdjust%sectd         = sgOfRef%sectd
       sgToAdjust%neutronicMixd = sgOfRef%neutronicMixd
    end if
  end subroutine adjustMix

  function interSgAr(sg,ar,pt1x,pt1y,pt2x,pt2y)
    type(t_segArc),intent(in)    :: sg,ar
    double precision,intent(out) :: pt1x,pt1y,pt2x,pt2y
    integer                      :: interSgAr

    double precision    :: prjx,prjy,d
    double precision    :: u,v,tmp,Ix,Iy,Jx,Jy
    integer :: IinS,IinA,JinS,JinA
    !renvoie le nombre de points d'intersection entre sg et ar,
    !et leurs coordonnees eventuelles
    !Si un seul point d'intersection, renvoi 1 si pt==I et -1 si pt==J
    !Si l'arc et le segment sont tangents, renvoie 2, et pt1=pt2

    interSgAr = 0
    pt1x=0.d0 ; pt1y=0.d0 ; pt2x=1.d0 ; pt2y=1.d0
    d = distance(ar%x,ar%y,sg%x,sg%y,sg%dx,sg%dy,prjx,prjy)
    if (.not. isEqualConst(d,ar%r)) then
       if (d>ar%r) then
          interSgAr = 0
       else
          !2 points d'intersection distincts I et J avec
          !I avant J sur la droite d'appuis du segment
          u=sg%dx-sg%x ; v=sg%dy-sg%y
          tmp=sqrt((ar%r*ar%r-d*d)/(u*u+v*v))
          u=tmp*u ; v=tmp*v
          Ix=prjx-u ; Iy=prjy-v ; IinS=isIn(Ix,Iy,sg) ; IinA=isIn(Ix,Iy,ar)
          Jx=prjx+u ; Jy=prjy+v ; JinS=isIn(Jx,Jy,sg) ; JinA=isIn(Jx,Jy,ar)
          if ((IinS/=0.and.IinA/=0).and.(IinS==2.or.IinA==2)) then
             interSgAr=1 ; pt1x=Ix ; pt1y=Iy
          end if
          if ((JinS/=0.and.JinA/=0).and.(JinS==2.or.JinA==2)) then
             if (interSgAr==1) then
                interSgAr=2 ; pt2x=Jx ; pt2y=Jy
             else
                interSgAr=-1 ; pt1x=Jx ; pt1y=Jy
             end if
          end if
       end if
    else
       Ix=prjx ; Iy=prjy ; IinS=isIn(Ix,Iy,sg) ; IinA=isIn(Ix,Iy,ar)
       if ((IinS/=0.and.IinA/=0).and.(IinS==2.or.IinA==2)) then
          interSgAr=2 ; pt1x=Ix ; pt1y=Iy ; pt2x=Ix ; pt2y=Iy
       end if
    end if
  end function interSgAr

  function interSgSg(sg1,sg2,ix,iy)
    type(t_segArc),intent(in)    :: sg1,sg2
    double precision,intent(out) :: ix,iy
    logical                      :: interSgSg

    double precision :: A1,B1,C1,A2,B2,C2,delta
    ix = 0.d0 ; iy = 0.d0
    A1=sg1%y-sg1%dy ; B1=sg1%dx-sg1%x ; C1=sg1%x*sg1%dy-sg1%dx*sg1%y
    A2=sg2%y-sg2%dy ; B2=sg2%dx-sg2%x ; C2=sg2%x*sg2%dy-sg2%dx*sg2%y
    delta = A1*B2-A2*B1
    interSgSg = .not. isEqualConst(delta,0.d0)
    if (.not. interSgSg) return
    delta = 1./delta
    ix = (B1*C2-B2*C1)*delta ; iy = (A2*C1-A1*C2)*delta
    interSgSg = (isIn(ix,iy,sg1)==2 .and. isIn(ix,iy,sg2)==2)
  end function interSgSg

  function translateAndTurnTriSg(cx,cy,turn,sg)
    double precision,intent(in) :: cx,cy
    integer,intent(in)          :: turn
    type(t_segArc),intent(in)   :: sg
    type(t_segArc)              :: translateAndTurnTriSg

    translateAndTurnTriSg = sg
    select case(turn)
    case(1)
       translateAndTurnTriSg%x  =  sg%x  + cx
       translateAndTurnTriSg%y  =  sg%y  + cy
       translateAndTurnTriSg%dx =  sg%dx + cx
       translateAndTurnTriSg%dy =  sg%dy + cy
    case(2)
       translateAndTurnTriSg%x  =  5.d-1*sg%x  - sqrt3_2d*sg%y  + cx
       translateAndTurnTriSg%y  =  5.d-1*sg%y  + sqrt3_2d*sg%x  + cy
       translateAndTurnTriSg%dx =  5.d-1*sg%dx - sqrt3_2d*sg%dy + cx
       translateAndTurnTriSg%dy =  5.d-1*sg%dy + sqrt3_2d*sg%dx + cy       
    case(3)
       translateAndTurnTriSg%x  = -5.d-1*sg%x  - sqrt3_2d*sg%y  + cx
       translateAndTurnTriSg%y  = -5.d-1*sg%y  + sqrt3_2d*sg%x  + cy
       translateAndTurnTriSg%dx = -5.d-1*sg%dx - sqrt3_2d*sg%dy + cx
       translateAndTurnTriSg%dy = -5.d-1*sg%dy + sqrt3_2d*sg%dx + cy       
    case(4)
       translateAndTurnTriSg%x  = -sg%x  + cx
       translateAndTurnTriSg%y  = -sg%y  + cy
       translateAndTurnTriSg%dx = -sg%dx + cx
       translateAndTurnTriSg%dy = -sg%dy + cy
    case(5)
       translateAndTurnTriSg%x  = -5.d-1*sg%x  + sqrt3_2d*sg%y  + cx
       translateAndTurnTriSg%y  = -5.d-1*sg%y  - sqrt3_2d*sg%x  + cy
       translateAndTurnTriSg%dx = -5.d-1*sg%dx + sqrt3_2d*sg%dy + cx
       translateAndTurnTriSg%dy = -5.d-1*sg%dy - sqrt3_2d*sg%dx + cy       
    case(6)
       translateAndTurnTriSg%x  =  5.d-1*sg%x  + sqrt3_2d*sg%y  + cx
       translateAndTurnTriSg%y  =  5.d-1*sg%y  - sqrt3_2d*sg%x  + cy
       translateAndTurnTriSg%dx =  5.d-1*sg%dx + sqrt3_2d*sg%dy + cx
       translateAndTurnTriSg%dy =  5.d-1*sg%dy - sqrt3_2d*sg%dx + cy       
    case(7)
       translateAndTurnTriSg%mixg  =  sg%mixd
       translateAndTurnTriSg%mixd  =  sg%mixg
       translateAndTurnTriSg%x  =  sg%x  + cx
       translateAndTurnTriSg%y  = -sg%y  + cy
       translateAndTurnTriSg%dx =  sg%dx + cx
       translateAndTurnTriSg%dy = -sg%dy + cy
    case(8)
       translateAndTurnTriSg%mixg  =  sg%mixd
       translateAndTurnTriSg%mixd  =  sg%mixg
       translateAndTurnTriSg%x  =  5.d-1*sg%x  - sqrt3_2d*sg%y  + cx
       translateAndTurnTriSg%y  = -5.d-1*sg%y  - sqrt3_2d*sg%x  + cy
       translateAndTurnTriSg%dx =  5.d-1*sg%dx - sqrt3_2d*sg%dy + cx
       translateAndTurnTriSg%dy = -5.d-1*sg%dy - sqrt3_2d*sg%dx + cy       
    case(9)
       translateAndTurnTriSg%mixg  =  sg%mixd
       translateAndTurnTriSg%mixd  =  sg%mixg
       translateAndTurnTriSg%x  = -5.d-1*sg%x  - sqrt3_2d*sg%y  + cx
       translateAndTurnTriSg%y  =  5.d-1*sg%y  - sqrt3_2d*sg%x  + cy
       translateAndTurnTriSg%dx = -5.d-1*sg%dx - sqrt3_2d*sg%dy + cx
       translateAndTurnTriSg%dy =  5.d-1*sg%dy - sqrt3_2d*sg%dx + cy       
    case(10)
       translateAndTurnTriSg%mixg  =  sg%mixd
       translateAndTurnTriSg%mixd  =  sg%mixg
       translateAndTurnTriSg%x  = -sg%x  + cx
       translateAndTurnTriSg%y  =  sg%y  + cy
       translateAndTurnTriSg%dx = -sg%dx + cx
       translateAndTurnTriSg%dy =  sg%dy + cy
    case(11)
       translateAndTurnTriSg%mixg  =  sg%mixd
       translateAndTurnTriSg%mixd  =  sg%mixg
       translateAndTurnTriSg%x  = -5.d-1*sg%x  + sqrt3_2d*sg%y  + cx
       translateAndTurnTriSg%y  =  5.d-1*sg%y  + sqrt3_2d*sg%x  + cy
       translateAndTurnTriSg%dx = -5.d-1*sg%dx + sqrt3_2d*sg%dy + cx
       translateAndTurnTriSg%dy =  5.d-1*sg%dy + sqrt3_2d*sg%dx + cy       
    case(12)
       translateAndTurnTriSg%mixg  =  sg%mixd
       translateAndTurnTriSg%mixd  =  sg%mixg
       translateAndTurnTriSg%x  =  5.d-1*sg%x  + sqrt3_2d*sg%y  + cx
       translateAndTurnTriSg%y  = -5.d-1*sg%y  + sqrt3_2d*sg%x  + cy
       translateAndTurnTriSg%dx =  5.d-1*sg%dx + sqrt3_2d*sg%dy + cx
       translateAndTurnTriSg%dy = -5.d-1*sg%dy + sqrt3_2d*sg%dx + cy       
    end select
  end function translateAndTurnTriSg

  function interArcArc(clus,ring,aClus,bClus,aRing,bRing,revert)
    type(t_segArc),intent(in)    :: ring,clus
    double precision,intent(out) :: aClus,bClus,aRing,bRing
    logical,intent(out)          :: revert !les arcs sont dans le meme sens?
    integer                      :: interArcArc

    double precision :: d,maxD,minD,aC,aR,d2,rr2,rc2
    integer          :: inClus,inRing
    logical          :: aIn,bIn

    !donne le nombre de points d'intersection entre clus et ring
    !et les angles eventuels d'intersection dans clus et ring
    !avec pour reference l'arc clus (pour les point designes)
    interArcArc = 0
    aClus = 0.d0 ; bClus = 0.d0
    aRing = 0.d0 ; bRing = 0.d0

    d = longVect(ring%x-clus%x,ring%y-clus%y)
    maxD = ring%r + clus%r + epsilon
    minD = abs(ring%r-clus%r) - epsilon
    d2 = d**2 ; rr2 = ring%r**2 ; rc2 = clus%r**2
    revert = (rr2>(d2+rc2)).or.(rc2>(d2+rr2))

    if (d>(maxD+epsilon).or.d<(minD-epsilon)) return !pas d'intersection

    if (isEqualConst(d,maxD).or.isEqualConst(d,minD)) then
       !tangence entre les deux cercles d'appuis
       aClus  = calculeAngle(clus%x,clus%y,ring%x,ring%y)
       if (revert) aClus = angleNormal(aClus - pi_c)
       inRing = isIn(clus%x+clus%r*cos(aClus),clus%y+clus%r*sin(aClus),ring)
       aRing  = calculeAngle(ring%x,ring%y,clus%x,clus%y)
       inClus = isIn(ring%x+ring%r*cos(aRing),ring%y+ring%r*sin(aRing),clus)
       if (inRing==2 .and. inClus==2) interArcArc = 1 !dans les arcs
       return
    end if

    !il y a intersection reelle des deux cercles d'appuis
    ! l'angle a1 adjacent a r1 est determinable a l'aide de la formule
    !  cos(a1) = (d\B2+r1\B2-r2\B2)/(2*d*r1) (idem mutatis-mutandis pour a2)
    aC = acos((d2+rc2-rr2)/(2*d*clus%r))
    aClus = angleNormal(calculeAngle(clus%x,clus%y,ring%x,ring%y) - aC)
    bClus = angleNormal(calculeAngle(clus%x,clus%y,ring%x,ring%y) + aC)
    aR = acos((d2+rr2-rc2)/(2*d*ring%r))
    aRing = angleNormal(calculeAngle(ring%x,ring%y,clus%x,clus%y) - aR)
    bRing = angleNormal(calculeAngle(ring%x,ring%y,clus%x,clus%y) + aR)
    aIn = (isAngleInArc(aClus,clus)==2) .and. (isAngleInArc(aRing,ring)==2)
    bIn = (isAngleInArc(bClus,clus)==2) .and. (isAngleInArc(bRing,ring)==2)
    if (aIn .and. bIn) then
       interArcArc = 2
    else if (aIn) then
       interArcArc = 1
    else if (bIn) then
       interArcArc = 1
       aClus = bClus
       aRing = bRing
    end if
  end function interArcArc

  function giveExtremalsAngles(sa,cx,cy,ao,ae)
    type(t_segArc),intent(in)    :: sa
    double precision,intent(in)  :: cx,cy
    double precision,intent(out) :: ao,ae
    logical                      :: giveExtremalsAngles

    double precision :: ox,oy,ex,ey

    select case(sa%typ)
    case(tcer)
       ! on sort avec une valeur false
       ao = 0.d0 ; ae=0.d0 ; giveExtremalsAngles = .false.
       return
    case(tarc)
       call extremitesArc(sa,ox,oy,ex,ey)
    case(tseg)
       ox = sa%x ; oy = sa%y ; ex = sa%dx ; ey = sa%dy
    end select
    ao = angleNormal(calculeAngle(cx,cy,ox,oy))
    ae = angleNormal(calculeAngle(cx,cy,ex,ey))
    giveExtremalsAngles = .not.( (isEqualConst(cx,ox).and.isEqualConst(cy,oy))&
         .or.(isEqualConst(cx,ex).and.isEqualConst(cy,ey)) )
  end function giveExtremalsAngles
  
  subroutine drawSegArc(fileNbr,szSA,withNodes,drawMix)
    integer,intent(in) :: fileNbr,szSA
    logical,intent(in) :: withNodes,drawMix

    type(t_segArc) :: sa
    integer        :: i
    real           :: cx,cy,angl,lx,ly,tailleNbr

    g_psp_bBoxXmin = 1.e10
    g_psp_bBoxYmin = 1.e10
    g_psp_bBoxXmax = -1.e10
    g_psp_bBoxYmax = -1.e10
    !recuperation des donnees permettants le centrage
    do i = 1,szSA
       sa = tabSegArc(i)%p
       if (sa%typ==tseg) then
          g_psp_bBoxXmin = min(real(sa%x),real(sa%dx),g_psp_bBoxXmin)
          g_psp_bBoxYmin = min(real(sa%y),real(sa%dy),g_psp_bBoxYmin)
          g_psp_bBoxXmax = max(real(sa%x),real(sa%dx),g_psp_bBoxXmax)
          g_psp_bBoxYmax = max(real(sa%y),real(sa%dy),g_psp_bBoxYmax)
       else
          g_psp_bBoxXmin = min(real(sa%x-sa%r),g_psp_bBoxXmin)
          g_psp_bBoxYmin = min(real(sa%y-sa%r),g_psp_bBoxYmin)
          g_psp_bBoxXmax = max(real(sa%x+sa%r),g_psp_bBoxXmax)
          g_psp_bBoxYmax = max(real(sa%y+sa%r),g_psp_bBoxYmax)
       end if
    end do
    lx = 0.1 * ( g_psp_bBoxXmax - g_psp_bBoxXmin )
    ly = 0.1 * ( g_psp_bBoxYmax - g_psp_bBoxYmin )
    g_psp_bBoxXmin = g_psp_bBoxXmin - lx
    g_psp_bBoxYmin = g_psp_bBoxYmin - ly
    g_psp_bBoxXmax = g_psp_bBoxXmax + lx
    g_psp_bBoxYmax = g_psp_bBoxYmax + ly
    tailleNbr = min(595./(g_psp_bBoxXmax-g_psp_bBoxXmin), &
         842./(g_psp_bBoxYmax-g_psp_bBoxYmin))
    tailleNbr = 3.6 / tailleNbr
    !impression
    call PSINIT(fileNbr,.true.)
    do i = 1,szSA
       sa = tabSegArc(i)%p
       if (sa%typ==tseg) then
          call line(sa%x,sa%y,sa%dx,sa%dy)
          cx=real((sa%dx+sa%x)*0.5d0) ; cy=real((sa%dy+sa%y)*0.5d0)
          angl = real(calculeAngle(sa%x,sa%y,sa%dx,sa%dy)*rad2deg-90.d0)
          if (withNodes .and. drawMix) then
             call keknum(cx,cy,tailleNbr,real(sa%nodeg),angl,-1,2)
             call keknum(cx,cy,tailleNbr,real(sa%noded),angl,-1,0)
          else if (drawMix) then
             call keknum(cx,cy,tailleNbr,real(sa%neutronicMixg),angl,-1,2)
             call keknum(cx,cy,tailleNbr,real(sa%neutronicMixd),angl,-1,0)
          end if
       else if (sa%typ==tcer) then
          call CIRCLE(real(sa%x),real(sa%y),real(sa%r),.false.)
          cx=real(sa%x+sa%r) ; cy=real(sa%y)
          if (withNodes .and. drawMix) then
             call keknum(cx,cy,tailleNbr,real(sa%nodeg),0.,-1,2)
             call keknum(cx,cy,tailleNbr,real(sa%noded),0.,-1,0)
          else if (drawMix) then
             call keknum(cx,cy,tailleNbr,real(sa%neutronicMixg),0.,-1,2)
             call keknum(cx,cy,tailleNbr,real(sa%neutronicMixd),0.,-1,0)
          end if
       else
          call ARC(real(sa%x),real(sa%y),real(sa%r), &
               & real(sa%a*rad2deg),real(sa%b*rad2deg))
          if (sa%b>sa%a) then
             angl=real((sa%b+sa%a)*0.5d0)
          else
             angl=real((sa%b+sa%a)*0.5d0+pi_c)
          end if
          cx=real(sa%x+cos(angl)*sa%r) ; cy=real(sa%y+sin(angl)*sa%r)
          angl=real(angl*rad2deg)
          if (withNodes .and. drawMix) then
             call keknum(cx,cy,tailleNbr,real(sa%nodeg),angl,-1,2)
             call keknum(cx,cy,tailleNbr,real(sa%noded),angl,-1,0)
          else if (drawMix) then
             call keknum(cx,cy,tailleNbr,real(sa%neutronicMixg),angl,-1,2)
             call keknum(cx,cy,tailleNbr,real(sa%neutronicMixd),angl,-1,0)
          end if
       end if
    end do
    call PLOTND()
  end subroutine drawSegArc

  subroutine coupeCercle(indDeb,szSA,nbSeg,typ)
    integer,intent(in)           :: indDeb,nbSeg,typ
    integer,intent(inout)        :: szSA

    type(t_segArc)   :: sg,ar
    double precision :: pt1x,pt1y,pt2x,pt2y,angl1,angl2,midx,midy,tmp
    integer          :: i,j,k,tailleAv,tailleAp,nbPtInter
    type(segArcArrayBis),dimension(:),allocatable :: tmpTabSegArc

    ! pour programmation defensive
    integer          :: taille_table_temp 

    !RQ: indDeb est l'indice du dernier element du tableau qui
    ! ne doive pas etre pris en compte. nbSeg est le nombre
    ! de segments paralleles aux axes, et typ correspond au type (3,4,6,ou...)
    ! CS-IV : Correction des commentaires d'origine
    ! CS-IV : indDeb : indice du dernier element a ne pas prendre en compte 
    ! CS-IV :          (point de depart)
    ! CS-IV : nbSeg : nombre de SA a prendre en charge
    ! CS-IV : szSA : nombre total de SA dans le tableau

    !preparation du tableau de travail
    tailleAp = szSA-indDeb

    ! CS-IV : pourquoi ce 25 ?(on considere que la decoupe ne va pas 
    ! CS-IV : accroitre le nombre de segments de plus de 25 fois
    ! CS-IV : ce coef devrait etre reflechi et dependre du nbr de cercles

    ! pour programmation defensive
    taille_table_temp = 25*tailleAp
    allocate(tmpTabSegArc(taille_table_temp))
    !allocate(tmpTabSegArc(25*tailleAp))
    do i = 1,size(tmpTabSegArc)
       nullify(tmpTabSegArc(i)%p)
    end do
    !copie des segments et arc construits dans le tableau
    !temporaire
    ! CS-IV : et suppression dans le tableau initial
    do i = 1,tailleAp
       allocate(tmpTabSegArc(i)%p)
       tmpTabSegArc(i)%p = tabSegArc(indDeb+i)%p
       tmpTabSegArc(i)%keep = .true.
       deallocate(tabSegArc(indDeb+i)%p)
    end do

    szSa = indDeb
    !on coupe tout les elements possible presents, en ne
    !gardant pas ceux qui sont a couper, mais seulement les
    !morceaux obtenus
    !pour i->segment, pour j->arc ou cercle
    do
       tailleAv = tailleAp
       i = 0
       do
          i = i+1
          if (.not. tmpTabSegArc(i)%keep) then
             if (i/=tailleAv ) cycle
             exit
          end if
          sg = tmpTabSegArc(i)%p
          if (sg%typ/=tseg) then
             if (i/=tailleAv ) cycle
             exit
          end if
          ! CS-IV : le SA tmpTabSegArc(i) est un segment
          do j = 1,tailleAv
             if ((j==i).or.(.not. tmpTabSegArc(j)%keep)) cycle
             ar = tmpTabSegArc(j)%p
             if (ar%typ==tseg) cycle
             ! CS-IV : le SA tmpTabSegArc(j) est un arc
             nbPtInter = interSgAr(sg,ar,pt1x,pt1y,pt2x,pt2y)
             if (nbPtInter==2) then
                !!deux points d'intersection : 
                ! CS-IV : on obtient 3 segments et 2 arcs
                tmpTabSegArc(i)%keep=.false.
                tmpTabSegArc(j)%keep=.false.
                ! => 3 segments (qui gardent les caracteristiques sect*)
                do k = 1,3
                   tailleAp=tailleAp+1
                   ! pour programmation defensive
                   if (tailleAP > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine CoupeCercle (1)")
                   allocate(tmpTabSegArc(tailleAp)%p)
                   tmpTabSegArc(tailleAp)%p=sg
                end do
                tmpTabSegArc(tailleAp-2)%p%dx=pt1x
                tmpTabSegArc(tailleAp-2)%p%dy=pt1y
                tmpTabSegArc(tailleAp-1)%p%x=pt1x
                tmpTabSegArc(tailleAp-1)%p%y=pt1y
                tmpTabSegArc(tailleAp-1)%p%dx=pt2x
                tmpTabSegArc(tailleAp-1)%p%dy=pt2y
                tmpTabSegArc(tailleAp)%p%x=pt2x
                tmpTabSegArc(tailleAp)%p%y=pt2y
                tmpTabSegArc(tailleAp-2)%p%mixg=ar%mixd
                tmpTabSegArc(tailleAp-2)%p%mixd=ar%mixd
                tmpTabSegArc(tailleAp-1)%p%mixg=ar%mixg
                tmpTabSegArc(tailleAp-1)%p%mixd=ar%mixg
                tmpTabSegArc(tailleAp)%p%mixg=ar%mixd
                tmpTabSegArc(tailleAp)%p%mixd=ar%mixd
                ! elimination d'un eventuel segment nul
                do k = 0,2
                   tmpTabSegArc(tailleAp-k)%keep= .not. &
                        & ( (isEqual( tmpTabSegArc(tailleAp-k)%p%x , &
                        &             tmpTabSegArc(tailleAp-k)%p%dx) ) .and. &
                        &   (isEqual( tmpTabSegArc(tailleAp-k)%p%y , &
                        &             tmpTabSegArc(tailleAp-k)%p%dy) ) )
                end do
                angl1=calculeAngle(ar%x,ar%y,pt1x,pt1y)
                angl2=calculeAngle(ar%x,ar%y,pt2x,pt2y)
                !travail sur les arcs
                if (ar%typ==tcer) then
                   !c'etait un cercle complet
                   if (isEqualAngl(angl1,angl2)) then !tangence
                      tailleAp=tailleAp+1
                      ! pour programmation defensive
                      if (tailleAP > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine CoupeCercle (2)")
                      allocate(tmpTabSegArc(tailleAp)%p)
                      tmpTabSegArc(tailleAp)%p=ar
                      tmpTabSegArc(tailleAp)%keep=.true.
                      tmpTabSegArc(tailleAp)%p%typ=tarc
                      tmpTabSegArc(tailleAp)%p%a=angl1
                      tmpTabSegArc(tailleAp)%p%b=angl1
                      !positionnement des sect*
                      if (estAGauche(sg%x,sg%y,sg%dx,sg%dy,ar%x,ar%y)) then
                         !cercle completement a gauche de sg
                         tmpTabSegArc(tailleAp)%p%sectg=sg%sectg
                         tmpTabSegArc(tailleAp)%p%sectd=sg%sectg
                      else
                         !cercle completement a droite de sg
                         tmpTabSegArc(tailleAp)%p%sectg=sg%sectd
                         tmpTabSegArc(tailleAp)%p%sectd=sg%sectd
                      end if
                   else !pas tangence
                      !un arc de chaque cote de sg (1\B0 a drt, 2\B0 a gch)
                      do k = 1,2
                         tailleAp=tailleAp+1
                         ! pour programmation defensive
                         if (tailleAP > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine CoupeCercle (3)")
                         allocate(tmpTabSegArc(tailleAp)%p)
                         tmpTabSegArc(tailleAp)%p=ar
                         tmpTabSegArc(tailleAp)%keep=.true.
                         tmpTabSegArc(tailleAp)%p%typ=tarc
                      end do
                      tmpTabSegArc(tailleAp-1)%p%a=angl1
                      tmpTabSegArc(tailleAp-1)%p%b=angl2
                      tmpTabSegArc(tailleAp-1)%p%sectg=sg%sectd
                      tmpTabSegArc(tailleAp-1)%p%sectd=sg%sectd
                      tmpTabSegArc(tailleAp)%p%a=angl2
                      tmpTabSegArc(tailleAp)%p%b=angl1
                      tmpTabSegArc(tailleAp)%p%sectg=sg%sectg
                      tmpTabSegArc(tailleAp)%p%sectd=sg%sectg
                   end if
                else !c'etait deja un arc de cercle
                   do k = 1,3
                      tailleAp=tailleAp+1
                      ! pour programmation defensive
                      if (tailleAP > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine CoupeCercle (4)")
                      allocate(tmpTabSegArc(tailleAp)%p)
                      tmpTabSegArc(tailleAp)%p=ar
                   end do
                   tmpTabSegArc(tailleAp-2)%p%b=angl1
                   !on test si on est bien dans le bon sens
                   if (isIn(pt2x,pt2y,tmpTabSegArc(tailleAp-2)%p)==2) then
                      !on est a l'envers (ouverture a droite)
                      tmpTabSegArc(tailleAp-2)%p%b=angl2
                      tmpTabSegArc(tailleAp-1)%p%a=angl2
                      tmpTabSegArc(tailleAp-1)%p%b=angl1
                      tmpTabSegArc(tailleAp)%p%a=angl1
                      !ajustement des sect*
                      tmpTabSegArc(tailleAp-2)%p%sectg=sg%sectd
                      tmpTabSegArc(tailleAp-2)%p%sectd=sg%sectd
                      tmpTabSegArc(tailleAp-1)%p%sectg=sg%sectg
                      tmpTabSegArc(tailleAp-1)%p%sectd=sg%sectg
                      tmpTabSegArc(tailleAp)%p%sectg=sg%sectd
                      tmpTabSegArc(tailleAp)%p%sectd=sg%sectd
                   else !on est a l'endroit (ouverture a gauche)
                      tmpTabSegArc(tailleAp-1)%p%a=angl1
                      tmpTabSegArc(tailleAp-1)%p%b=angl2
                      tmpTabSegArc(tailleAp)%p%a=angl2
                      !ajustement des sect*
                      tmpTabSegArc(tailleAp-2)%p%sectg=sg%sectg
                      tmpTabSegArc(tailleAp-2)%p%sectd=sg%sectg
                      tmpTabSegArc(tailleAp-1)%p%sectg=sg%sectd
                      tmpTabSegArc(tailleAp-1)%p%sectd=sg%sectd
                      tmpTabSegArc(tailleAp)%p%sectg=sg%sectg
                      tmpTabSegArc(tailleAp)%p%sectd=sg%sectg
                   end if
                   do k = 0,2
                      tmpTabSegArc(tailleAp-k)%keep=.not. &
                           & ( isEqualAngl( tmpTabSegArc(tailleAp-k)%p%a , &
                           &                tmpTabSegArc(tailleAp-k)%p%b ) )
                   end do
                end if
                i=0
                exit
             else if (abs(nbPtInter)==1) then
                !! un seul point d'intersection
                ! CS-IV : on obtient 2 segments & 2 arcs
                tmpTabSegArc(i)%keep=.false.
                tmpTabSegArc(j)%keep=.false.
                do k = 1,2
                   tailleAp=tailleAp+1
                   ! pour programmation defensive
                   if (tailleAP > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine CoupeCercle (5)")
                   allocate(tmpTabSegArc(tailleAp)%p)
                   tmpTabSegArc(tailleAp)%p=sg
                end do
                tmpTabSegArc(tailleAp-1)%p%dx=pt1x
                tmpTabSegArc(tailleAp-1)%p%dy=pt1y
                tmpTabSegArc(tailleAp)%p%x=pt1x
                tmpTabSegArc(tailleAp)%p%y=pt1y
                if (nbPtInter==1) then
                   !intersection en I (origne de sg a l'exterieur de ar)
                   tmpTabSegArc(tailleAp-1)%p%mixg=ar%mixd
                   tmpTabSegArc(tailleAp-1)%p%mixd=ar%mixd
                   tmpTabSegArc(tailleAp)%p%mixg=ar%mixg
                   tmpTabSegArc(tailleAp)%p%mixd=ar%mixg
                else
                   !intersection en J (origne de sg a l'interieur de ar)
                   tmpTabSegArc(tailleAp-1)%p%mixg=ar%mixg
                   tmpTabSegArc(tailleAp-1)%p%mixd=ar%mixg
                   tmpTabSegArc(tailleAp)%p%mixg=ar%mixd
                   tmpTabSegArc(tailleAp)%p%mixd=ar%mixd
                end if
                do k = 0,1
                   tmpTabSegArc(tailleAp-k)%keep=.not. &
                        & ( ( isEqual( tmpTabSegArc(tailleAp-k)%p%x , &
                        &              tmpTabSegArc(tailleAp-k)%p%dx ) ) .and.&
                        &   ( isEqual( tmpTabSegArc(tailleAp-k)%p%y , &
                        &              tmpTabSegArc(tailleAp-k)%p%dy ) ) )
                end do
                angl1=calculeAngle(ar%x,ar%y,pt1x,pt1y)
                if (ar%typ==tcer) then
                   !ar etait un cercle
                   tailleAp=tailleAp+1
                   ! pour programmation defensive
                   if (tailleAP > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine CoupeCercle (6)")
                   allocate(tmpTabSegArc(tailleAp)%p)
                   tmpTabSegArc(tailleAp)%p=ar
                   tmpTabSegArc(tailleAp)%p%typ=tarc
                   tmpTabSegArc(tailleAp)%p%a=angl1
                   tmpTabSegArc(tailleAp)%p%b=angl1
                   tmpTabSegArc(tailleAp)%keep=.true.
                   !pas d'ajustement des sect*, car l'arc n'est ni d'un cote
                   !ni de l'autre
                else
                   do k = 1,2
                      tailleAp=tailleAp+1
                      ! pour programmation defensive
                      if (tailleAP > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine CoupeCercle (7)")
                      allocate(tmpTabSegArc(tailleAp)%p)
                      tmpTabSegArc(tailleAp)%p=ar
                   end do
                   tmpTabSegArc(tailleAp-1)%p%b=angl1
                   tmpTabSegArc(tailleAp)%p%a=angl1
                   !ajustement des sect*
                   if (nbPtInter==1) then
                      !intersection en I (origne de sg a l'exterieur de ar)
                      tmpTabSegArc(tailleAp-1)%p%sectg=sg%sectg
                      tmpTabSegArc(tailleAp-1)%p%sectd=sg%sectg
                      tmpTabSegArc(tailleAp)%p%sectg=sg%sectd
                      tmpTabSegArc(tailleAp)%p%sectd=sg%sectd
                   else
                      !intersection en J (origne de sg a l'interieur de ar)
                      tmpTabSegArc(tailleAp-1)%p%sectg=sg%sectd
                      tmpTabSegArc(tailleAp-1)%p%sectd=sg%sectd
                      tmpTabSegArc(tailleAp)%p%sectg=sg%sectg
                      tmpTabSegArc(tailleAp)%p%sectd=sg%sectg
                   end if
                   do k = 0,1
                      tmpTabSegArc(tailleAp-k)%keep=.not. &
                           & ( isEqualAngl( tmpTabSegArc(tailleAp-k)%p%a , &
                           &                tmpTabSegArc(tailleAp-k)%p%b ) )
                   end do
               end if
                i=0
                exit
             end if
          end do
          if (i==tailleAv) exit
       end do
       if ((tailleAp==tailleAv) .and. (i==tailleAv)) exit
    end do

    !on elimine les element en dehors des limites de la cellule, et on remet le
    !milieu exterieur a la cellule a fooMix
    !Remarque : les elements tmpTabSegArc(1:nbSeg)%p sont les segments
    !initiaux paralleles aux bords du domaine. Ils peuventt etre a jetter
    !(keep==.false.), mais ils servent tout de meme de reference. Pour savoir
    !si ils sont au bord de la cellule, on test si le milieux est fooMix.
    !Si oui, on elimine tous les element qui sont de ce cote.
    if (typ==tRec .or. typ==tHex) then
       do i = 1,nbSeg
          sg = tmpTabSegArc(i)%p
          do j = 1,tailleAp
             ar = tmpTabSegArc(j)%p
             if (ar%typ==tseg) then
                if (.not. &
                     (estColi(sg%dx-sg%x,sg%dy-sg%y,ar%dx-ar%x,ar%dy-ar%y) &
                     .and. pointsAlignes(sg%x,sg%y,sg%dx,sg%dy,ar%x,ar%y)) &
                     ) cycle
                if (sg%mixd==fooMix) then
                   tmpTabSegArc(j)%p%mixd=fooMix
                else if (sg%mixg==fooMix) then
                   tmpTabSegArc(j)%p%mixg=fooMix
                end if
                cycle
             end if
             if ((j==i) .or. (.not. tmpTabSegArc(j)%keep)) cycle
             if (ar%b>ar%a) then
                angl1=(ar%b+ar%a)*0.5d0
             else
                angl1=(ar%b+ar%a)*0.5d0+pi_c
             end if
             !point milieu de l'arc
             midx=ar%x+ar%r*cos(angl1)
             midy=ar%y+ar%r*sin(angl1)
             !sinus de l'angle (OE,OM)
             tmp = sin( calculeAngle(sg%x,sg%y,midx,midy) - &
                  &     calculeAngle(sg%x,sg%y,sg%dx,sg%dy) )
             if (sg%mixd==fooMix) then !on garde a gauche
                tmpTabSegArc(j)%keep=(tmp>0.).or.(isEqualAngl(ar%a,ar%b).and. &
                     & estAGauche(sg%x,sg%y,sg%dx,sg%dy,ar%x,ar%y))
             else if (sg%mixg==fooMix) then  !on garde a droite
                tmpTabSegArc(j)%keep=(tmp<0.).or.(isEqualAngl(ar%a,ar%b).and. &
                     & estADroite(sg%x,sg%y,sg%dx,sg%dy,ar%x,ar%y))
             end if
          end do
       end do
    else
       call XABORT("G2S : type of geometrie not supported")
    end if

    !elimination des segments nuls eventuellement restants
    do i = 1,tailleAp
       sg = tmpTabSegArc(i)%p
       if (sg%typ==tseg) tmpTabSegArc(i)%keep=tmpTabSegArc(i)%keep .and. &
            .not.(isEqual(tmpTabSegArc(i)%p%x,tmpTabSegArc(i)%p%dx) .and. &
            isEqual(tmpTabSegArc(i)%p%y,tmpTabSegArc(i)%p%dy))
    end do

    !on recupere dans le tableau general les elements restants
    do i = 1,tailleAp
       if (.not. tmpTabSegArc(i)%keep) cycle
       szSa = szSa+1
       allocate(tabSegArc(szSa)%p)
       tabSegArc(szSa)%p = tmpTabSegArc(i)%p
    end do

    !on nettoie le tableau temporaire
    do i = 1,tailleAp
       deallocate(tmpTabSegArc(i)%p) ; nullify(tmpTabSegArc(i)%p)
    end do
    deallocate(tmpTabSegArc)
  end subroutine coupeCercle

  subroutine splitSegsForSect(indDeb,szSA)
    integer,intent(in)    :: indDeb
    integer,intent(inout) :: szSA

    type(t_segArc) :: sgi,sgj
    integer        :: i,j

    ! pour programmation defensive
    integer :: taille_table_tabSegArc

    !! permet de spliter les segments en bordure de domaine,
    !! losqu'ils sont interceptes par les segments delimitants
    !! la sectorisation. Si on a intersection, c'est toujours a
    !! l'interieuure d'un segment de bordure, et au bout d'un segment
    !! de secteur.                      -->^----->          <-----^<--
    !! Dans ce cas, on split ainsi: (1)  i |  szSA  ou: (2)  szSA | i 
    
    ! pour programmation defensive
    taille_table_tabSegArc = size(tabSegArc)
    
    i = indDeb
    do
       i = i + 1
       if (i>szSA) exit
       sgi=tabSegArc(i)%p
       if (sgi%typ/=tseg) cycle
       do j = indDeb+1,szSA
          if (j==i) cycle
          sgj=tabSegArc(j)%p
          if (sgj%typ/=tseg) cycle
          if (PointsAlignes(sgi%x,sgi%y,sgi%dx,sgi%dy,sgj%dx,sgj%dy).and.&
               (isIn(sgj%dx,sgj%dy,sgi)==2).and. &
               .not.PointsAlignes(sgi%x,sgi%y,sgi%dx,sgi%dy,sgj%x,sgj%y)) then
             szSA=szSA+1
             ! pour programmation defensive
             if (szSA > taille_table_tabSegArc) &
                  call XABORT("G2S : memory problem in routine splitSegsForSect")
             allocate(tabSegArc(szSA)%p)
             tabSegArc(szSA)%p = sgi
             tabSegArc(i)%p%dx = sgj%dx   ; tabSegArc(i)%p%dy = sgj%dy
             tabSegArc(szSA)%p%x = sgj%dx ; tabSegArc(szSA)%p%y = sgj%dy
            
             if (estAGaucheStrict(sgj%x,sgj%y,sgj%dx,sgj%dy,sgi%x,sgi%y)) then
                !! cas (1)
                tabSegArc(i)%p%sectg = sgj%sectg
                tabSegArc(i)%p%sectd = sgj%sectg
                tabSegArc(szSA)%p%sectg = sgj%sectd
                tabSegArc(szSA)%p%sectd = sgj%sectd
             else
                !! cas (2)
                tabSegArc(i)%p%sectg = sgj%sectd
                tabSegArc(i)%p%sectd = sgj%sectd
                tabSegArc(szSA)%p%sectg = sgj%sectg
                tabSegArc(szSA)%p%sectd = sgj%sectg
             end if
             sgi = tabSegArc(i)%p
          end if
       end do
    end do
  end subroutine splitSegsForSect

  subroutine majSectori(indDeb,szSA,sectori,typGeo,cx,cy)
    double precision, parameter :: PI = 3.141592653589793d0
    integer,intent(in)          :: indDeb,szSA,sectori,typGeo
    double precision,intent(in) :: cx,cy
    !met a jour les secteurs pour les elements

    double precision :: ao,ae !angles origine et extremite
    integer          :: i,j,nbSect,oIn,eIn
    logical          :: goodAngles
    double precision,dimension(9) :: limit

    if ((typGeo/=tRec).and.(typGeo/=tHex)) &
        call XABORT("G2S: internal error with typGeo in subroutine majSectori")
    !creation des zones angulaires
    nbSect = 0
    select case(sectori)
    case(S_not)
       return !rien a faire
    case(S_X_tot)
       if (typGeo == tRec) then
          nbSect = 4
          limit(1) = pi_2_c*5.d-1 ; limit(2) = pi_2_c*1.5d0
          limit(3) = -limit(2)    ; limit(4) = -limit(1)
          limit(5) = limit(1)
       else
          nbSect = 6
          limit(1) = 0.d0 ; limit(2) = pi_3_c    ; limit(3) = pi_3_c*2.d0
          limit(4) = pi_c ; limit(5) = -limit(3) ; limit(6) = -limit(2)
          limit(7) = limit(1)
       end if
    case(S_T_tot)
       nbSect = 4
       limit(1) = 0.d0 ; limit(2) = pi_2_c
       limit(3) = pi_c ; limit(4) = -limit(2)
       limit(5) = limit(1)
    case(S_TX_tot)
       nbSect = 8
       limit(1) = 0.d0      ; limit(2) = pi_2_c*5.d-1
       limit(3) = pi_2_c    ; limit(4) = pi_2_c*1.5d0
       limit(5) = pi_c      ; limit(6) = -limit(4)
       limit(7) = -limit(3) ; limit(8) = -limit(2)
       limit(9) = limit(1)
    case(S_TXS_tot)
       nbsect = 8
       limit(1) = PI/8. 
       do i=1, 7 
          limit(i+1) = limit(i) + PI/4. 
       enddo
       limit(9) = limit(1) 
   case(S_WM_tot)
       nbsect = 8
       limit(1) = PI/8. 
       do i=1, 7 
          limit(i+1) = limit(i) + PI/4. 
       enddo
       limit(9) = limit(1) 
    case default
       call XABORT("G2S: internal error with sectori in subroutine majSectori")
    end select
    !traitement des elements
    do i = indDeb+1,szSA
       if (tabSegArc(i)%p%typ == tcer) cycle !on ne s'interesse pas aux cercles
       !on recupere les angles des bouts
       goodAngles = giveExtremalsAngles(tabSegArc(i)%p,cx,cy,ao,ae)
       if (.not.goodAngles) cycle !une des extremites est le centre 
       !  => on ne s'en occupe pas
       do j = 1,nbSect
          !boucle sur les secteurs
          oIn = isAngleInInterval(ao,limit(j),limit(j+1))
          eIn = isAngleInInterval(ae,limit(j),limit(j+1))
          if ((oIn==0) .or. (eIn==0)) then
             !pas dans le secteur
             cycle !on passe au secteur suivant
          else if ((oIn==2) .or. (eIn==2)) then
             !on est completement dans le secteur
             tabSegArc(i)%p%sectg = j
             tabSegArc(i)%p%sectd = j
             exit !on sort de la boucle en j
          else if (oIn/=eIn) then
             !element occupant le secteur angulaire total => idem cas precedent
             tabSegArc(i)%p%sectg = j
             tabSegArc(i)%p%sectd = j
             exit !on sort de la boucle en j
          else
             !les 2 extremites sont sur le meme separateur
             ! => on ne s'en occupe pas
             exit !on sort de la boucle en j
          end if
       end do
    end do
  end subroutine majSectori

  subroutine addSegsAndClean(sizeSA)
    integer,intent(inout) :: sizeSA

    integer :: i,j,sizeTmp
    integer :: oj,ej !test l'appartenance a sgi des bouts de sgj
    type(t_segArc) :: sgi,sgj
    type(segArcArrayBis),dimension(:),allocatable :: tmpTab

    ! pour programmation defensive
    integer          :: taille_table_temp, taille_table_tabSegArc

    !copie dans un tableau temporaire des segments
    !et nettoyage du tableau global
    ! pour programmation defensive
    taille_table_temp = sizeSA*10
    taille_table_tabSegArc = size(tabSegArc)

    allocate(tmpTab(taille_table_temp))
    do i = 1,sizeSA
       allocate(tmpTab(i)%p)
       tmpTab(i)%p=tabSegArc(i)%p
       tmpTab(i)%keep=.true.
       deallocate(tabSegArc(i)%p)
       nullify(tabSegArc(i)%p)
    end do

    sizeTmp=sizeSA
    i=0
    do
       i = i+1
       if (i>sizeTmp) exit
       if (.not. tmpTab(i)%keep) cycle
       if (tmpTab(i)%p%typ/=tseg) cycle
       sgi=tmpTab(i)%p !c'est bien un segment a garder pour le moment
       j = 0
       do
          j = j+1
          if (j>sizeTmp) exit
          if (j==i) cycle
          if (.not. tmpTab(j)%keep) cycle
          if (tmpTab(j)%p%typ/=tseg) cycle
          sgj=tmpTab(j)%p
          if (.not. estColineaire(sgi,sgj)) cycle
          if (.not. pointsAlignes(sgi%x,sgi%y,sgi%dx,sgi%dy,sgj%x,sgj%y)) cycle
          !on renverse le sens de sgj si besoin
          if (.not. isSameWay(sgi,sgj)) sgj=turnBackSide(tmpTab(j)%p)
          oj=isIn(sgj%x,sgj%y,sgi) ; ej=isIn(sgj%dx,sgj%dy,sgi)
          select case(ej) !seuls cas 0,2,3 utiles pour ej
          case(0) !seuls cas 0,1,2 utiles pour oj
             select case(oj)
             case(0) !si isIn(sgi%x,sgi%y,sgj)/=2 rien sinon, sgi dans sgj
                if (isIn(sgi%x,sgi%y,sgj)/=2) cycle
                call adjustMix(tmpTab(i)%p,sgj)
                sizeTmp=sizeTmp+1 
                ! pour programmation defensive
                if (sizeTmp > taille_table_temp) &
                     call XABORT("G2S : memory problem in addSegsAndClean (1)")
                allocate(tmpTab(sizeTmp)%p)
                tmpTab(sizeTmp)%keep=.true. ; tmpTab(sizeTmp)%p=sgj
                tmpTab(j)%p%dx=sgi%x ; tmpTab(j)%p%dy=sgi%y
                tmpTab(sizeTmp)%p%x=sgi%dx ; tmpTab(sizeTmp)%p%y=sgi%dy
             case(1) !origines confondues et sgi dans sgj
               call adjustMix(tmpTab(i)%p,sgj)
                tmpTab(j)%p%x=sgi%dx ; tmpTab(j)%p%y=sgi%dy
             case(2) !sgi et sgj s'intersectent reellement (sgj au dessus)
                sizeTmp=sizeTmp+1
                ! pour programmation defensive
                if (sizeTmp > taille_table_temp) &
                     call XABORT("G2S : memory problem in addSegsAndClean (2)")
                allocate(tmpTab(sizeTmp)%p)
                tmpTab(sizeTmp)%keep=.true. ; tmpTab(sizeTmp)%p=sgi
                tmpTab(i)%p%dx=sgj%x ; tmpTab(i)%p%dy=sgj%y
                tmpTab(sizeTmp)%p%x=sgj%x ; tmpTab(sizeTmp)%p%y=sgj%y
                tmpTab(sizeTmp)%p%dx=sgi%dx ; tmpTab(sizeTmp)%p%dy=sgi%dy
                call adjustMix(tmpTab(sizeTmp)%p,sgj)
                tmpTab(j)%p%x=sgi%dx ; tmpTab(j)%p%y=sgi%dy
             end select
          case(2) !seuls cas 0,1,2 utiles pour oj
             select case(oj)
             case(0) !sgi et sgj s'intersectent reellement (sgj en dessous)
                sizeTmp=sizeTmp+1 
                ! pour programmation defensive
                if (sizeTmp > taille_table_temp) &
                     call XABORT("G2S : memory problem in addSegsAndClean (3)")
                allocate(tmpTab(sizeTmp)%p)
                tmpTab(sizeTmp)%keep=.true. ; tmpTab(sizeTmp)%p=sgi
                tmpTab(i)%p%x=sgj%dx ; tmpTab(i)%p%y=sgj%dy
                tmpTab(sizeTmp)%p%x=sgi%x ; tmpTab(sizeTmp)%p%y=sgi%y
                tmpTab(sizeTmp)%p%dx=sgj%dx ; tmpTab(sizeTmp)%p%dy=sgj%dy
                call adjustMix(tmpTab(sizeTmp)%p,sgj)
                tmpTab(j)%p%dx=sgi%x ; tmpTab(j)%p%dy=sgi%y
             case(1) !origines confondues et sgj dans sgi
                call adjustMix(tmpTab(j)%p,sgi)
                tmpTab(i)%p%x=sgj%dx ; tmpTab(i)%p%y=sgj%dy
             case(2) !sgj dans sgi
                call adjustMix(tmpTab(j)%p,sgi)
                sizeTmp=sizeTmp+1 
                ! pour programmation defensive
                if (sizeTmp > taille_table_temp) &
                     call XABORT("G2S : memory problem in addSegsAndClean (4)")
                allocate(tmpTab(sizeTmp)%p)
                tmpTab(sizeTmp)%keep=.true. ; tmpTab(sizeTmp)%p=sgi
                tmpTab(i)%p%dx=sgj%x ; tmpTab(i)%p%dy=sgj%y
                tmpTab(sizeTmp)%p%x=sgj%dx ; tmpTab(sizeTmp)%p%y=sgj%dy
             end select
          case(3) !seuls cas 0,1,2 utiles pour oj
             select case(oj)
             case(0) !extremites confondues et sgi dans sgj
                call adjustMix(tmpTab(i)%p,sgj)
                tmpTab(j)%p%dx=sgi%x ; tmpTab(j)%p%dy=sgi%y
             case(1) !sgi et sgj confondus
                call adjustMix(tmpTab(i)%p,sgj)
                tmpTab(j)%keep=.false.
             case(2) !extremites confondues et sgj dans sgi
                call adjustMix(tmpTab(j)%p,sgi)
                tmpTab(i)%p%dx=sgj%x ; tmpTab(i)%p%dy=sgj%y
             end select
          end select
       end do
    end do

    do i = 1,sizeTmp
       if (tmpTab(i)%keep) then
          if (tmpTab(i)%p%typ==tseg) then
             if (isEqual(tmpTab(i)%p%x,tmpTab(i)%p%dx).and. &
                  isEqual(tmpTab(i)%p%y,tmpTab(i)%p%dy)) tmpTab(i)%keep=.false.
          end if
       end if
    end do

    !recopie dans le tableau global des elements a conserver
    sizeSA=0
    do i = 1,sizeTmp
       if (tmpTab(i)%keep) then
          sizeSA=sizeSA+1
          ! pour programmation defensive
          if (sizeSA > taille_table_tabSegArc) &
               call XABORT("G2S : memory problem in addSegsAndClean (5)")
          allocate(tabSegArc(sizeSA)%p)
          tabSegArc(sizeSA)%p=tmpTab(i)%p
       end if
       deallocate(tmpTab(i)%p)
       nullify(tmpTab(i)%p)
    end do
    deallocate(tmpTab)

  end subroutine addSegsAndClean

  subroutine cutClusters(nbRing,nbClus,szSa)
    integer,intent(in)    :: nbRing,nbClus
    integer,intent(inout) :: szSa

    integer          :: i,j,szTmp,nbInter
    double precision :: aClus,bClus,aRing,bRing
    logical          :: revert,eqA,eqB
    type(t_segArc)   :: ring,clus
    type(segArcArrayTer),dimension(:),allocatable :: tmpTabAr

    ! pour programmation defensive
    integer :: taille_table_temp

    taille_table_temp = nbRing*nbClus*2 !ca me semble suffisant (note de l'auteur !)
    allocate(tmpTabAr(taille_table_temp)) 
    szTmp = 0

    !initialisation du tableau temporaire
    !recopie des cluster et des anneaux dans l'ordre inverse de leur
    !creation => du plus grand vers le plus petit
    do i = 1,nbClus
       szTmp = szTmp + 1
       allocate(tmpTabAr(szTmp)%p)
       tmpTabAr(szTmp)%keep = .true.
       tmpTabAr(szTmp)%cl = .true.
       tmpTabAr(szTmp)%p = tabSegArc(szSa)%p
       deallocate(tabSegArc(szSa)%p)
       nullify(tabSegArc(szSa)%p)
       szSa = szSa - 1
    end do
    do i = 1,nbRing
       szTmp = szTmp + 1
       allocate(tmpTabAr(szTmp)%p)
       tmpTabAr(szTmp)%keep = .true.
       tmpTabAr(szTmp)%cl = .false.
       tmpTabAr(szTmp)%p = tabSegArc(szSa)%p
       deallocate(tabSegArc(szSa)%p)
       nullify(tabSegArc(szSa)%p)
       szSa = szSa - 1
    end do

    i=0
    do
       i = i+1
       if (i>szTmp) exit
       if (.not. (tmpTabAr(i)%keep .and. tmpTabAr(i)%cl)) cycle
       clus = tmpTabAr(i)%p
       j = 0
       do
          j = j+1
          if (j>szTmp) exit
          if ((.not. tmpTabAr(j)%keep) .or. tmpTabAr(j)%cl) cycle
          ring = tmpTabAr(j)%p
          nbInter = interArcArc(clus,ring,aClus,bClus,aRing,bRing,revert)
          select case(nbInter)
          case(2)
             if (clus%typ==tcer) then
                !demi cercle interieur
                tmpTabAr(i)%p%typ = tarc
                tmpTabAr(i)%p%mixd = ring%mixg
                tmpTabAr(i)%p%a = aClus
                tmpTabAr(i)%p%b = bClus
                clus = tmpTabAr(i)%p
                !demi cercle exterieur
                szTmp = szTmp + 1
                ! programmation defensive
                if (sztmp > taille_table_temp) &
                     call XABORT("G2S : memory problem in routine cutClusters (1)")
                allocate(tmpTabAr(szTmp)%p)
                tmpTabAr(szTmp)%keep = .true.
                tmpTabAr(szTmp)%cl = .true.
                tmpTabAr(szTmp)%p = clus
                tmpTabAr(szTmp)%p%mixd = ring%mixd
                tmpTabAr(szTmp)%p%a = bClus
                tmpTabAr(szTmp)%p%b = aClus                 
             else
                eqA = isEqualAngl(clus%a,aClus)
                eqB = isEqualAngl(clus%b,bClus)
                !debut
                if (.not.eqA) then
                   szTmp = szTmp + 1
                   ! programmation defensive
                   if (sztmp > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine cutClusters (2)")
                   allocate(tmpTabAr(szTmp)%p)
                   tmpTabAr(szTmp)%keep = .true.
                   tmpTabAr(szTmp)%cl = .true.
                   tmpTabAr(szTmp)%p = clus
                   tmpTabAr(szTmp)%p%mixd = ring%mixd
                   tmpTabAr(szTmp)%p%b = aClus
                end if
                !fin
                if (.not.eqB) then
                   szTmp = szTmp + 1
                   ! programmation defensive
                   if (sztmp > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine cutClusters (3)")
                   allocate(tmpTabAr(szTmp)%p)
                   tmpTabAr(szTmp)%keep = .true.
                   tmpTabAr(szTmp)%cl = .true.
                   tmpTabAr(szTmp)%p = clus
                   tmpTabAr(szTmp)%p%mixd = ring%mixd
                   tmpTabAr(szTmp)%p%a = bClus
                end if
                !milieu
                if ((.not.eqA).or.(.not.eqB)) then
                   tmpTabAr(i)%p%a = aClus
                   tmpTabAr(i)%p%b = bClus
                   tmpTabAr(i)%p%mixd = ring%mixg
                   clus = tmpTabAr(i)%p
                end if
             end if
             if (ring%typ==tcer) then
                tmpTabAr(j)%p%typ = tarc
                tmpTabAr(j)%p%a = bRing
                tmpTabAr(j)%p%b = aRing
                ring = tmpTabAr(j)%p
             else
                tmpTabAr(j)%keep = .false.
                eqA = isEqualAngl(ring%a,aRing)
                eqB = isEqualAngl(ring%b,bRing)
                !premier bout d'arc
                if (.not.eqA) then
                   szTmp = szTmp + 1
                   ! programmation defensive
                   if (sztmp > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine cutClusters (4)")
                   allocate(tmpTabAr(szTmp)%p)
                   tmpTabAr(szTmp)%keep = .true.
                   tmpTabAr(szTmp)%cl = .false.
                   tmpTabAr(szTmp)%p = ring
                   tmpTabAr(szTmp)%p%b = aRing
                end if
                !second bout d'arc
                if (.not.eqB) then
                   szTmp = szTmp + 1
                   ! programmation defensive
                   if (sztmp > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine cutClusters (5)")
                   allocate(tmpTabAr(szTmp)%p)
                   tmpTabAr(szTmp)%keep = .true.
                   tmpTabAr(szTmp)%cl = .false.
                   tmpTabAr(szTmp)%p = ring
                   tmpTabAr(szTmp)%p%a = bRing
                end if
                deallocate(tmpTabAr(j)%p)
                nullify(tmpTabAr(j)%p)
             end if
          case(1)
             !a priori, tel que l'algo est fait, le cas ne se produit que si
             !le cluster est tangent a un anneau
             !travail sur l'anneau
             if (ring%typ==tcer) then
                tmpTabAr(j)%p%typ = tarc
                tmpTabAr(j)%p%a = aRing
                tmpTabAr(j)%p%b = aRing
                ring = tmpTabAr(j)%p
             else
                tmpTabAr(j)%keep = .false.
                eqA = isEqualAngl(ring%a,aRing)
                eqB = isEqualAngl(ring%b,aRing)
                !premier bout d'arc
                if (.not.eqA) then
                   szTmp = szTmp + 1
                   ! programmation defensive
                   if (sztmp > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine cutClusters (6)")
                   allocate(tmpTabAr(szTmp)%p)
                   tmpTabAr(szTmp)%keep = .true.
                   tmpTabAr(szTmp)%cl = .false.
                   tmpTabAr(szTmp)%p = ring
                   tmpTabAr(szTmp)%p%b = aRing
                end if
                !second bout d'arc
                if (.not.eqB) then
                   szTmp = szTmp + 1
                   ! programmation defensive
                   if (sztmp > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine cutClusters (7)")
                   allocate(tmpTabAr(szTmp)%p)
                   tmpTabAr(szTmp)%keep = .true.
                   tmpTabAr(szTmp)%cl = .false.
                   tmpTabAr(szTmp)%p = ring
                   tmpTabAr(szTmp)%p%a = aRing
                end if
                deallocate(tmpTabAr(j)%p)
                nullify(tmpTabAr(j)%p)
             end if
             !travail sur le cluster
             if (clus%typ==tcer) then
                tmpTabAr(i)%p%typ = tarc
                tmpTabAr(i)%p%a = aClus
                tmpTabAr(i)%p%b = aClus
                clus = tmpTabAr(i)%p
                if (revert) then !cluster a l'exterieur de l'anneau
                   tmpTabAr(i)%p%mixd = ring%mixd
                else              !cluster a l'interieur de l'anneau
                   tmpTabAr(i)%p%mixd = ring%mixg
                end if
             else ! le cluster a deja ete coupe => tangence exterieur
                !   et derniere utilisation
                eqA = isEqualAngl(clus%a,aClus)
                eqB = isEqualAngl(clus%b,aClus)
                !debut
                if (.not.eqA) then
                   szTmp = szTmp + 1
                   ! programmation defensive
                   if (sztmp > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine cutClusters (8)")
                   allocate(tmpTabAr(szTmp)%p)
                   tmpTabAr(szTmp)%keep = .true.
                   tmpTabAr(szTmp)%cl = .true.
                   tmpTabAr(szTmp)%p = clus
                   tmpTabAr(szTmp)%p%mixd = ring%mixd
                   tmpTabAr(szTmp)%p%b = aClus
                end if
                !fin
                if (.not.eqB) then
                   szTmp = szTmp + 1
                   ! programmation defensive
                   if (sztmp > taille_table_temp) &
                        call XABORT("G2S : memory problem in routine cutClusters (9)")
                   allocate(tmpTabAr(szTmp)%p)
                   tmpTabAr(szTmp)%keep = .true.
                   tmpTabAr(szTmp)%cl = .true.
                   tmpTabAr(szTmp)%p = clus
                   tmpTabAr(szTmp)%p%mixd = ring%mixd
                   tmpTabAr(szTmp)%p%a = aClus
                end if
                !milieu
                if ((.not.eqA).or.(.not.eqB)) then
                   tmpTabAr(i)%keep = .false.
                   deallocate(tmpTabAr(i)%p)
                   nullify(tmpTabAr(i)%p)
                   i = 0
                end if
             end if

          end select
       end do
    end do
    !recopie des resultats
    do i = 1,szTmp
       if (.not. tmpTabAr(i)%keep) cycle
       szSa = szSa + 1
       allocate(tabSegArc(szSa)%p)
       tabSegArc(szSa)%p = tmpTabAr(i)%p
       deallocate(tmpTabAr(i)%p)
       nullify(tmpTabAr(i)%p)
    end do
    deallocate(tmpTabAr)
  end subroutine cutClusters
  
  subroutine PrintTabSegArc(szSA)
    integer, intent(in) :: szSA
    type(T_segArc) :: sa
    integer :: i

    write(*,10) szSA

    do i = 1, szSA
       sa = tabSegArc(i)%p
       if (sa%typ == tseg) then
          write(*,20) i, sa%x, sa%y, sa%dx, sa%dy
       elseif (sa%typ == tarc) then
          write(*,30) i, sa%x, sa%y, sa%r, sa%a, sa%b
       elseif (sa%typ == tcer) then
          write(*,35) i, sa%x, sa%y, sa%r
       else
          write(*,38) i, sa%typ, sa%x, sa%y
       endif
       write(*,40) sa%sectg, sa%sectd
       write(*,50) sa%mixg, sa%mixd
       write(*,60) sa%nodeg, sa%noded
       write(*,70) sa%indcellpg, sa%indcellpd
       write(*,80) sa%neutronicMixg,sa%neutronicMixd
    enddo

10  format("Nombre de SegArc :", i4)
20  format("SegArc n. ", i6," de type segment : ",/, &
           "|-> origin/extrem : (", f7.4, ";", f7.4,")/(", f7.4, ";", f7.4,")")
30  format("SegArc n. ", i6," de type arc :",/, &
           "|-> centre/rayon/angles : (", f7.4,";",f7.4,")/", f7.4,"/",f7.4,";", f7.4)
35  format("SegArc n. ", i6," de type cercle :",/, &
           "|-> centre/rayon : (", f7.4,";",f7.4,")/", f7.4)
38  format("SegArc n. ", i6," de type inconnu :",/, &
           "|-> type/origine/ : (", i6,"/",f7.4,";",f7.4,")")
40 format( "|-> sectg sectd  : ", i6,2x,i6)
50 format( "|-> mixg mixd  : ", i6,2x,i6)
60 format( "|-> nodeg noded  : ", i6,2x,i6)
70 format( "|-> IndCellPg IndCellPd  : ", i6,2x,i6)
80 format( "|-> neutronicMixg neutronicMixd  : ", i6,2x,i6)
  end subroutine PrintTabSegArc

  subroutine PrintSegArc(i)
    integer, intent(in) ::i
    type(T_segArc) :: sa
    
    sa = tabSegArc(i)%p
    if (sa%typ == tseg) then
       write(*,20) i, sa%x, sa%y, sa%dx, sa%dy
     elseif (sa%typ == tarc) then
       write(*,30) i, sa%x, sa%y, sa%r, sa%a, sa%b
    elseif (sa%typ == tcer) then
       write(*,35) i, sa%x, sa%y, sa%r
    else
       write(*,38) i, sa%typ, sa%x, sa%y
    endif
    write(*,40) sa%sectg, sa%sectd
    write(*,50) sa%mixg, sa%mixd
    write(*,60) sa%nodeg, sa%noded
    write(*,70) sa%indcellpg, sa%indcellpd
    write(*,80) sa%neutronicMixg,sa%neutronicMixd
    
20  format("SegArc n. ", i6," de type segment : ",/, &
         "|-> origin/extrem : (", f7.4, ";", f7.4,")/(", f7.4, ";", f7.4,")")
30  format("SegArc n. ", i6," de type arc :",/, &
         "|-> centre/rayon/angles : (", f7.4,";",f7.4,")/", f7.4,"/",f7.4,";", f7.4)
35  format("SegArc n. ", i6," de type cercle :",/, &
         "|-> centre/rayon : (", f7.4,";",f7.4,")/", f7.4)
38  format("SegArc n. ", i6," de type inconnu :",/, &
         "|-> type/origine/ : (", i6,"/",f7.4,";",f7.4,")")
40  format( "|-> sectg sectd  : ", i6,2x,i6)
50  format( "|-> mixg mixd  : ", i6,2x,i6)
60  format( "|-> nodeg noded  : ", i6,2x,i6)
70  format( "|-> IndCellPg IndCellPd  : ", i6,2x,i6)
80  format( "|-> neutronicMixg neutronicMixd  : ", i6,2x,i6)
  end subroutine PrintSegArc

end module segArc
